#ifndef LIBGALSA_H
#define LIBGALSA_H

#include <glib.h>

struct GalsaDescription;
struct GalsaHeader;
struct GalsaComponent;
struct GalsaComponentPort;
struct GalsaComponentGUIOption;
struct GalsaComponentImplementation;
struct GalsaComponentImplementationRef;
struct GalsaComponentImplementationChannel;
struct GalsaComponentImplementationChannelPort;
struct GalsaComponentImplementationGenerationMethod;

enum ChanSense
{ ACTIVE, PASSIVE };
enum ChanDir
{ PULL, PUSH, PUSHPULL, NONE_SYNC };
enum NetDir
{ INPUT, OUTPUT, INOUT };

/* Stuff defined in libgalsa.c
static char *__ChanSenseStrings[] = {"active", "passive", NULL};
static char *__ChanDirStrings[] = {"pull", "push", "pushpull", "none_sync", NULL};
static char *__NetDirStrings[] = {"input", "output", "inout", NULL};
*/
extern char *__ChanSenseStrings[];
extern char *__ChanDirStrings[];
extern char *__NetDirStrings[];

struct GalsaDescription
{
    struct GalsaHeader *header;
    struct GalsaMainCircuit *mainCircuit;
    GList *components;          // List of GalsaComponent
};

struct GalsaHeader
{
    char *filename;
};

struct GalsaMainCircuit
{
    char *component;
    char *implementation;

    // Post-computed values
    struct GalsaDescription *parent;
};

struct GalsaComponent
{
    char *name;
    char *desc;
    GList *ports;               // List of GalsaComponentPort
    GList *GUIoptions;          // List of GalsaComponentGUIOption
    GList *implementations;     // List of GalsaComponentImplementation
    char *currently_edited_impl;

    // Post-computed values
    struct GalsaDescription *parent;
};

struct GalsaComponentPort
{
    char *name;
    enum
    { CHANNEL, NET } type;

    union
    {

        struct
        {
            enum ChanSense sense;
            enum ChanDir direction;
            int data_blocks;
            int data_block_width;
            int control_width;
            int array_size;
            //      char *dataType;
        } channel;

        struct
        {
            enum NetDir direction;
            int width;
        } net;

    };
};

struct GalsaComponentGUIOption
{
    char *name;
    char *value;
};

struct GalsaComponentImplementation
{
    char *name;
    char *desc;
    enum
    { SUB_CIRCUIT, SINGLE_ELEMENT } type;

    // This part is used if type==SUB_CIRCUIT
    // We don't use a union (yet) because of an issue with galsa-ide (properties plugin)
    struct
    {
        GList *components;      // List of GalsaComponentImplementationRef
        GList *channels;        // List of GalsaComponentImplementationChannel
        GList *wires;           // List of GalsaComponentImplementationWire
        GList *combinations;    // List of GalsaComponentImplementationCombination
    } sub_circuit;

    // This part is used if type==SINGLE_ELEMENT
    struct
    {
        char *file;
        char *procedure;
        GList *args;            // List of GalsaComponentImplementationArg
        struct GalsaComponentImplementationGenerationMethod *generationMethod;
		char *target;
    } single_element;

    // Post-computed values
    struct GalsaComponent *parent;
};

struct GalsaComponentImplementationRef
{
	gboolean is_converter;
    char *id;
    char *impl;
    char *componentName;

    // Post-computed values
    struct GalsaComponentImplementation *parent;
};

struct GalsaComponentImplementationChannel
{
    gboolean is_port;
    char *name;
    enum ChanDir direction;
    int data_blocks;
    int data_block_width;
    int control_width;
    int array_size;
    //  char *dataType;
    GList *ports;               // List of GalsaComponentImplementationChannelPort
	GList *converters;          // List of GalsaComponentImplementationChannelConverter

    // Post-computed values
    struct GalsaComponentImplementation *parent;
};

struct GalsaComponentImplementationChannelPort
{
    enum
    { INITIATOR, TARGET } type;
    char *comp_id;
    char *port_name;

    // Post-computed values
    struct GalsaComponentImplementationChannel *parent;
//    struct GalsaComponentImplementationRef *comp_impl_ref_ptr;
//    struct GalsaComponentPort *component_port_ptr;
};

struct GalsaComponentImplementationChannelConverter
{
    char *conv_id;

    // Post-computed values
};

struct GalsaComponentImplementationWire
{
    gboolean is_port;
    char *name;
    int width;
    char *fixed_value;
    GList *connections;         // List of GalsaComponentImplementationWireConnection

    // Post-computed values
    struct GalsaComponentImplementation *parent;
};

struct GalsaComponentImplementationWireConnection
{
    char *comp_id;
    char *port_name;

    // Post-computed values
    struct GalsaComponentImplementationWire *parent;
    //  struct GalsaComponent *component_ptr;
    //  struct GalsaComponentPort *component_port_ptr;
};

struct GalsaComponentImplementationCombination
{
    char *wire_name;
    GList *items;               // List of GalsaComponentImplementationCombinationItem
};

struct GalsaComponentImplementationCombinationItem
{
    char *name;
    char *wire_field;
};

struct GalsaComponentImplementationArg
{
    char *name;
    enum
    { DIRECT_PORT_CONNECTION, DIRECT_CONNECTION, CHANNEL_CONNECTION } type;

    union
    {
        struct
        {
            char *net_name;
        } direct_connection;

        struct
        {
            char *channel_name;
            char *channel_field;
        } channel_connection;
    };

};

struct GalsaComponentImplementationGenerationMethod
{
    enum
    { BALSA2BREEZE, BREEZE2IMPL } type;
    // ... TODO ...
};

/* Functions */
struct GalsaDescription *Galsa_NewBaseDescription (void);
struct GalsaDescription *Galsa_ParseFile (char *filename);
void Galsa_ProcessDescription (struct GalsaDescription *description);
void Galsa_Save (struct GalsaDescription *, char *filename);

/*
struct GalsaComponentImplementationImplementation
  *GalsaComponentImplementation_GetCurrentlyEditedImplementation (struct
								  GalsaComponentImplementation
								  *finalCircuit);
struct GalsaComponentImplementationImplementationItem
  *GalsaComponentImplementationImplementation_GetItemByCompName (struct
								 GalsaComponentImplementationImplementation
								 *impl,
								 char
								 *compName);

char *GalsaComponentImplementationRef_GetImplementationName (struct
								   GalsaComponentImplementationRef
								   *comp);
*/


// Operations on GalsaDescription

struct GalsaComponent *galsaDescription_GetComponentByName (struct GalsaDescription *description, char *comp_name);

char *galsaDescription_CreateUnusedComponentName (struct GalsaDescription *description, char *name_hint);
struct GalsaDescription *galsaDescription_Flatten (struct GalsaDescription *description);
void galsaDescription_Reduce (struct GalsaDescription *description);


// Operations on GalsaComponent

struct GalsaComponentImplementation *galsaComponent_GetImplementationByName (struct GalsaComponent *comp, char *impl_name);
struct GalsaComponentPort *galsaComponent_GetPortByName (struct GalsaComponent *comp, char *port_name);
char *galsaComponent_CreateUnusedImplementationName (struct GalsaComponent *comp);


// Operations on GalsaComponentImplementationRef

struct GalsaComponent *GalsaComponentImplementationRef_GetComponent (struct GalsaComponentImplementationRef *ref);


// Operations on GalsaComponentImplementationChannelPort

struct GalsaComponentImplementationRef *GalsaComponentImplementationChannelPort_GetRef (struct GalsaComponentImplementationChannelPort *port);


// Operations on GalsaComponentImplementationWireConnection

struct GalsaComponentImplementationRef *GalsaComponentImplementationWireConnection_GetRef (struct GalsaComponentImplementationWireConnection *con);

#endif
