/*
  The Balsa Asynchronous Hardware Synthesis System
  Copyright (C) 2002 Department of Computer Science
  The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#ifndef LIB_NETWORK_GRAPH_H
#define LIB_NETWORK_GRAPH_H

#include <glib.h>
#include "shape.h"

#define BEGINS_WITH(str1,str2) (!strncmp(str1,str2,strlen(str2)))


struct Vertex;

struct NetworkGraph
{
    // public members:
    struct Vertex *rootVertex;

    // private members:
    gboolean useControlForLayout;
};


struct Vertex
{
    char *label;                // Text displayed on the screen
    char *name;                 // Initial name of the real thing: procedure name, breeze component name, ...
    int type;
    void *data;
    struct Vertex *parent;
    GList *ports;               // List of struct Port

    gboolean isGroup;
    // 4 group properties:
    gboolean isShrunk;          // indicates if the group is not developed
    gboolean areSubgroupsExpanded;
    gboolean areChildsVisible;
    gboolean isHidden;
    GList *childs;  // List of struct Vertex

	struct Shape shape;
    double posX;
    double posY;
    double posZ;
    double sizeX;
    double sizeY;
	double inside_sizeX;
	double inside_sizeY;
    int color;

//    struct Position *balsaPosition;

    // Values refreshed every time a group is redrawn //developed/shrunk
	double current_inside_zoom;
	double current_inside_shiftX;
	double current_inside_shiftY;

    void *data2;
	void *internal_tmp_data;

    struct NetworkGraph *graph;
};

// The Port structure has 2 roles:
// - holding the visual item being drawn between a component and a channel
// - being the intermediate element when a group is developped, between the internal and the external worlds
// This structure is NOT necessarily in bijection with the component's I/Os. For example, one port can regroup 2 signals "inp" and "clk", and another one "out" and "clk".
struct Port
{
	struct Vertex *vertex;
//	struct Edge *internalEdge;
//	struct Edge *externalEdge;
	GList *internalEdges;   // List of struct Edge
	GList *externalEdges;   // List of struct Edge

    enum PortType
    { EMPTY_CIRCLE, FULL_CIRCLE, EMPTY_SQUARE } type;
    char *label;

	struct Shape shape;
    double posX;
    double posY;
    void *data;
};

enum EdgeDirection
{ EdgeDirection_Undefined, EdgeDirection_Sync, EdgeDirection_Pull,
    EdgeDirection_Push
};

#define EDGE_TYPE_WIREFORK_TREE 1
#define EDGE_TYPE_MODULE_MAINLOOP 2

struct Edge
{
//    struct Port *source;
//    struct Port *dest;
	struct Port *endpoint1;
	struct Port *endpoint2;

    gboolean isShrunk;
	GList *childs;    // List of struct Vertex -- I may limit this to 1 possible child for the moment

	enum { EDGE_SHAPE__NORMAL_LINE, EDGE_SHAPE__ONLY_BEGIN_AND_END_LINES } shape;

    int type;                   // 0=normal, 1=wirefork channel, 2=inside the main loop of a module
    int selectionLevel;
    enum EdgeDirection direction;
    int noweight;

    void *data;
	void *internal_tmp_data;
};

enum HandleType { HANDLE_CENTRE, HANDLE_RESIZE__TOP_LEFT, HANDLE_RESIZE__BOTTOM_RIGHT, HANDLE_XYPOINT, HANDLE_RESIZE__INSIDE_BOTTOM_RIGHT };
enum NetworkGraph_ElementType { NETWORK_GRAPH_ELEMENT__VERTEX, NETWORK_GRAPH_ELEMENT__PORT };


struct HandlePoint
{
	enum NetworkGraph_ElementType parentType;
	union {
		gpointer any;
		struct Vertex *vertex;
		struct Port *port;
	} parent;
	enum HandleType type;
};


typedef gboolean (*CALLBACK_VERTEX) (struct Vertex *);
typedef gboolean (*CALLBACK_EDGE) (struct Edge *);
typedef gboolean (*CALLBACK_HANDLE_POINT) (struct HandlePoint *);


/*************** Network graph ***************/
struct NetworkGraph *network_graph_new (void);

// Accessing graph properties
int *network_graph_get_dims (struct NetworkGraph *);

// Operations on graphs
void SaveGraph (struct NetworkGraph *graph, char *filename);
void LoadGraph (struct NetworkGraph *graph, char *filename);
void FullLayoutWithDot (struct NetworkGraph *graph);
struct Vertex *FindVertexGroupContainingPoint (struct NetworkGraph *graph, double X, double Y);
struct Vertex *FindVertexByName (struct NetworkGraph *graph, char *name, struct Vertex *parent, gboolean recursive);


/*************** Ports ***************/
struct Port *AddNewPortToVertex (struct Vertex *, enum PortType type, char *label);
struct Port *AddNewPortWithDataToVertex (struct Vertex *, enum PortType type, char *label, void *data);

// Accessing port properties
double GetPortCoordX (struct Port *);
double GetPortCoordY (struct Port *);
void *GetPortData (struct Port *);

void SetPortCoordX (struct Port *, double);
void SetPortCoordY (struct Port *, double);
void SetPortData (struct Port *, void *data);

// Operations on ports


/*************** Vertices ***************/
struct Vertex *AddNewVertex (char *name, gboolean isGroup, struct Vertex *parent);
struct Vertex *AddNewVertexWithData (char *name, gboolean isGroup, struct Vertex *parent, void *data);
//void UpdateVertexInternals (struct Vertex *vertex);

// Accessing Vertex properties
double GetVertexCoordX (struct Vertex *);
double GetVertexCoordY (struct Vertex *);
double GetVertexSizeX (struct Vertex *);
double GetVertexSizeY (struct Vertex *);
double GetVertexInsideSizeX (struct Vertex *);
double GetVertexInsideSizeY (struct Vertex *);
char *GetVertexName (struct Vertex *);
void *GetVertexData (struct Vertex *);

void SetVertexCoordX (struct Vertex *, double);
void SetVertexCoordY (struct Vertex *, double);
void SetVertexSizeX (struct Vertex *, double);
void SetVertexSizeY (struct Vertex *, double);
void SetVertexInsideSizeX (struct Vertex *, double);
void SetVertexInsideSizeY (struct Vertex *, double);
void SetVertexName (struct Vertex *, char *name);
void SetVertexData (struct Vertex *, void *data);

// Operations on Vertices
void ForEachVertex (struct NetworkGraph *graph, CALLBACK_VERTEX);
void ForEachVisibleVertex (struct NetworkGraph *graph, CALLBACK_VERTEX);
void ForEachVertexRecursive (struct Vertex *, CALLBACK_VERTEX fct_begin, CALLBACK_VERTEX fct_end);
void ForEachVisibleVertexRecursive (struct Vertex *, CALLBACK_VERTEX fct_begin, CALLBACK_VERTEX fct_end);
void ExpandGroupVertex (struct Vertex *);
void ShrinkGroupVertex (struct Vertex *);


/*************** Edges ***************/
struct Edge *AddNewEdge (struct Port *source, struct Port *dest, struct Vertex *container);
struct Edge *AddNewEdgeWithData (struct Port *source, struct Port *dest, struct Vertex *container, void *data);

// Accessing Edge properties
struct Port *GetSourcePort (struct Edge *);
struct Port *GetDestPort (struct Edge *);
struct Vertex *GetSourceVertex (struct Edge *);
struct Vertex *GetDestVertex (struct Edge *);
void *GetEdgeData (struct Edge *);
void SetEdgeData (struct Edge *, void *data);

// Operations on Edges
void ForEachVisibleEdge (struct NetworkGraph *graph, CALLBACK_EDGE);
void ForEachEdge (struct NetworkGraph *graph, CALLBACK_EDGE);



/*************** Handle points ***************/
struct HandlePoint *NewHandlePoint (enum NetworkGraph_ElementType parentType, gpointer parent, enum HandleType type);

// Accessing HandlePoint properties
double GetHandlePointCoordX (struct HandlePoint *);
double GetHandlePointCoordY (struct HandlePoint *);

// Operations on Handle points
void ForEachVisibleHandlePoint (struct NetworkGraph *graph, CALLBACK_HANDLE_POINT);






#if 0

#define isVertexVisible(v) (!v->isHidden && (!v->parent || (v->parent==v->parent->deepestVisibleParent && v->parent->areChildsVisible)))

//struct Vertex *AddNewVertex (char *name, gboolean isGroup, struct Vertex *parent);
//struct Vertex *AddNewVertexWithData (char *name, gboolean isGroup, struct Vertex *parent, void *data);
//struct Edge *AddNewEdge (struct Vertex *, struct Vertex *);
//struct Port *AddNewPortToVertex (struct Vertex *, enum PortType type, char *label);

void reset_libnetworkgraph (void);
void ComputeThreadGroups ();
void InitialiseVertexSizeRecursively (struct Vertex *);
void InitialiseVertexPositionRecursively (struct Vertex *, double x, double y);
void ComputeRealVertexSizeAndPositionRecursively (struct Vertex *);

//typedef gboolean (*CALLBACK_VERTEX) (struct Vertex *);
//typedef gboolean (*CALLBACK_EDGE) (struct Edge *);
typedef gboolean (*CALLBACK_EDGECON) (struct ConnectedVertex *);
typedef int (*COMPARISON_FCT) (void *, void *);

// Accessing Vertices
//void ForEachVertex (struct NetworkGraph *graph, CALLBACK_VERTEX);
//void ForEachVisibleVertex (struct NetworkGraph *graph, CALLBACK_VERTEX);
//void ForEachVertexRecursive (struct Vertex *, CALLBACK_VERTEX fct_begin, CALLBACK_VERTEX fct_end);
//void ForEachVisibleVertexRecursive (struct Vertex *, CALLBACK_VERTEX fct_begin, CALLBACK_VERTEX fct_end);
void ForEachVisibleVertexRecursiveAndPorts (struct Vertex *, CALLBACK_VERTEX fct_begin, CALLBACK_VERTEX fct_end);
void ForEachVertexConnectedTo (struct Vertex *, CALLBACK_VERTEX);
GList *GetEdgesConnectedTo (struct Vertex *);
//double GetVertexCoordX (struct Vertex *);
//double GetVertexCoordY (struct Vertex *);
//double GetVertexSizeX (struct Vertex *);
//double GetVertexSizeY (struct Vertex *);
double GetVertexRelSizeX (struct Vertex *);
double GetVertexRelSizeY (struct Vertex *);
//char *GetVertexName (struct Vertex *);
//void SetVertexName (struct Vertex *, char *name);
//void *GetVertexData (struct Vertex *);
//void SetVertexData (struct Vertex *, void *data);

struct Vertex *SearchVertex (struct NetworkGraph *graph, COMPARISON_FCT, void *);
void DestroyVertex (struct Vertex *);

//void ExpandGroupVertex (struct Vertex *);
//void ShrinkGroupVertex (struct Vertex *);

void RefreshAllConnectedVertices (struct NetworkGraph *graph);
void RecursivelyRefreshDeepestExpandedParent (struct Vertex *vertex, gboolean isVisible
  /*parentIsExpanded */ ,
  struct Vertex *deepestVisibleParent);

// Accessing Edges
void ForEachEdge (struct NetworkGraph *graph, CALLBACK_EDGE);
//void ForEachVisibleEdge (struct NetworkGraph *graph, CALLBACK_EDGECON);
void ForEachVisibleEdgeAndPorts (struct NetworkGraph *graph, CALLBACK_EDGECON);

void DestroyEdge (struct NetworkGraph *graph, struct Edge *edge);
void DestroyEdgeWithoutFree (struct NetworkGraph *graph, struct Edge *edge);
extern enum EdgeDirection SearchAndDestroyEdge_DestroyedEdgeDirection;
gboolean SearchAndDestroyEdge (struct NetworkGraph *graph, struct Vertex *, struct Vertex *);
//struct Vertex *GetSourceVertex (struct Edge *);
//struct Vertex *GetDestVertex (struct Edge *);

void PruneEmptyGroups (struct Vertex *);
void PruneGroupsWithOneGroupElement (struct Vertex *);
void PruneColoredVertices (struct Vertex *);


void UpdateVertexPositions (void);

void ExportDotFile (struct NetworkGraph *graph, char *filename);
void LayoutWithDot (struct Vertex *);
//void FullLayoutWithDot (struct NetworkGraph *graph);

#define LINE_WIDTH_FOR_NON_EMPTY_GROUP 20
#define LINE_WIDTH_FOR_EMPTY_GROUP 5
char *strdup_ApplyMaxWidth (char *str, int length, int returnType);
gboolean UpdateVertexLabel (struct Vertex *v);
void UpdateAllVertexLabels (void);

void SetVisibleRootVertex (struct NetworkGraph *graph, struct Vertex *v);

gboolean isEdgeVisible (struct Edge *edge);
void MakeEdgeVisible (struct Edge *edge);

void ScreenShot (struct NetworkGraph *graph, char *filename);

extern double MoveGroupContent_X;
extern double MoveGroupContent_Y;
gboolean MoveGroupContent (struct Vertex *v);

//struct Vertex *FindVertexGroupContainingPoint (struct NetworkGraph *graph, double X, double Y);


#endif
#endif
